#!/usr/bin/perl

#--------------------------------------------------------------
# debug_message ()
#
# This sub-function prints the string passed to it.
# This sub-function should be used only for debug messages.
#
# @param the messages string to be printed
#
#--------------------------------------------------------------
sub debug_message ($)
{
    if ($DEBUG eq $TRUE)
    {
	display_message (@_);
    }
}


#--------------------------------------------------------------
# display_message ()
#
# This sub-function prints the string passed to it.
# This sub-function should be used to convey information to
# users
#
# @param the messages string to be printed
#
#--------------------------------------------------------------
sub display_message ($)
{
    local ($str) = @_;
    print "$str";
}


#--------------------------------------------------------------
# usage ()
#
# Prints the Usage description
#--------------------------------------------------------------
sub usage ()
{
    debug_message ("Enter usage ()...\n");

    local ($USAGE) = "";
    local ($USAGE_SYNOPSIS) = "";
    local ($USAGE_OPTIONS) = "";
 
    $USAGE_SYNOPSIS = "	$0   [-h | --help] [-V | --version] [-v | --verbose] \n";
    $USAGE_SYNOPSIS = ($USAGE_SYNOPSIS) . "	     [-N | --groups <group,group,group,...>]\n";
    $USAGE_SYNOPSIS = ($USAGE_SYNOPSIS) . "	     [-n | --hosts  <host_name,host_name,host_name,...>]\n";
#    $USAGE_SYNOPSIS = ($USAGE_SYNOPSIS) . "	     [-s | --suppress_splash_screen]\n";
    $USAGE_SYNOPSIS = ($USAGE_SYNOPSIS) . "	     [command_specification]\n\n";	

    $USAGE_OPTIONS = "	-h, --help    : Output usage message. \n";
    $USAGE_OPTIONS = ($USAGE_OPTIONS) . "	-V, --version : Output version information.\n";
    $USAGE_OPTIONS = ($USAGE_OPTIONS) . "	-v, --verbose : Run in debug mode. \n";
    $USAGE_OPTIONS = ($USAGE_OPTIONS) . "	-N, --groups <group,group,group,...> : Overrides the groups and\n";
    $USAGE_OPTIONS = ($USAGE_OPTIONS) . "	clears hosts fields.\n";
    $USAGE_OPTIONS = ($USAGE_OPTIONS) . "	-n, --hosts  <host_name,host_name,host_name,...> : Overrides the Host names\n";
    $USAGE_OPTIONS = ($USAGE_OPTIONS) . "	and clears groups field.\n";
#    $USAGE_OPTIONS = ($USAGE_OPTIONS) . "	-s, --suppress_splash_screen : Hides splash screen at start up. \n";

    $USAGE = "Usage:\n";
    $USAGE = ($USAGE) . ($USAGE_SYNOPSIS); 
    $USAGE = ($USAGE) . ($USAGE_OPTIONS);

    # Use the message catalog
    #`dspmsg dcem.cat 10 \"$USAGE\"`

    display_message ($USAGE);

    debug_message ("Leave usage ().\n");
    exit (1);

}


#--------------------------------------------------------------
# run_dcem ()
#
# Executes the "djava" command with the appropriate arguments
#--------------------------------------------------------------
sub run_dcem ()
{
    debug_message ("Enter run_dcem ()...\n");

    local ($CMD) = ($DJAVA);
    
    if ($DEBUG eq $TRUE)
    {
	$CMD = ($CMD) . (" ") . ("-debug");
    }

    $CMD = ($CMD) . (" ") . ($JAVA_ARGS) . (" ") . ($JAVA_CLASS_NAME) . (" ") . ($DISTRIBUTED_CMD) . ($SYSTEM_DEFAULTS) . ($USER_DEFAULTS) . ($FACTORY_DEFAULTS) . ($USER_SETTINGS) . ($APPLICATION_MESSAGES);

    if ($SUPPRESS_SPLASH_SCREEN eq $FALSE)
    {
        $CMD = ($CMD) . ($SPLASH_SCREEN);
    }

    #display_message ("Executing: $CMD\n");
    debug_message ("Executing: $CMD\n");
    select (STDOUT);
    $| = 1;
    select (STDERR);
    $| = 1;
    exec ("$CMD");

    debug_message ("Leave run_dcem ().\n");
}


#--------------------------------------------------------------
# process_arguments ()
#
# Process the command line arguments passed to this script
#--------------------------------------------------------------
sub process_arguments ()
{
    debug_message ("Enter process_arguments ()...\n");

    local ($arg)	= "";
    local ($count)	= 0;
    local ($groupArgFound) = $FALSE;
    local ($hostArgFound) = $FALSE;
    local ($commandArgFound) = $FALSE;
    local ($versionArgFound) = $FALSE;
    local ($debugArgFound)   = $FALSE;
    
    $count = @ARGV;
    
    while ($count > 0)
    {
	debug_message ("Argument = $ARGV[0]\n");
	debug_message ("count = $count\n");
        
        if (($ARGV[0]=~/^-D/))
        {
	    $JAVA_ARGS = join(" ", ($JAVA_ARGS, $ARGV[0]));
	}
        elsif (($ARGV[0] !~ /^-/) && ($commandArgFound eq $FALSE))
	{
	    $commandArgFound=$TRUE;
	    $DISTRIBUTED_CMD = join(" ", ($DISTRIBUTED_CMD, $ARGV[0]));
	}
	elsif (($ARGV [$0] eq "--suppress_splash_screen") || ($ARGV [$0] eq "-s")) 
	{
            $SUPPRESS_SPLASH_SCREEN = $TRUE;
	}
        elsif ((($ARGV [$0] eq "--groups") || ($ARGV [$0] eq "-N")) && ($groupArgFound eq $FALSE))
        {
            $groupArgFound = $TRUE;
            $DISTRIBUTED_CMD = join(" ", ($DISTRIBUTED_CMD, $ARGV[0]));
	    shift (@ARGV); # discard the first argument in the array
            $count--;

            if (($count < 1) || (substr($ARGV[0],0,1) eq "-"))  # there must be group list argument
            {
                usage ();
            }
            else
	    {
		debug_message ("GROUPS = $ARGV[0]\n");
		$DISTRIBUTED_CMD = join(" ", ($DISTRIBUTED_CMD, $ARGV[0]));
	    }
        }
	elsif ((($ARGV [$0] eq "--hosts")  || ($ARGV [$0] eq "-n")) && ($hostArgFound eq $FALSE))
        {
            $hostArgFound = $TRUE;
            $DISTRIBUTED_CMD = join(" ", ($DISTRIBUTED_CMD, $ARGV[0]));
	    shift (@ARGV); # discard the first argument in the array
            $count--;

            if (($count < 1) || (substr($ARGV[0],0,1) eq "-"))  # there must be host list argument
            {
                usage ();
            }
            else
	    {
		debug_message ("HOSTS = $ARGV[0]\n");
		$DISTRIBUTED_CMD = join(" ", ($DISTRIBUTED_CMD, $ARGV[0]));
	    }
        }
	elsif ((($ARGV [$0] eq "--version") || ($ARGV [$0] eq "-V")) && ($versionArgFound eq $FALSE))
	{
		$versionArgFound=$TRUE;
		display_message($VERSION);
		
	}
	elsif (($ARGV [$0] eq "--help") || ($ARGV [$0] eq "-h")) 
	{
		usage();
	}
	else # other arguments
	{
	    usage ();
	}
	
        # put a space after the last appended flag
        $DISTRIBUTED_CMD = join("", ($DISTRIBUTED_CMD, " "));

	shift (@ARGV); # discard the first argument in the array
	$count--;
    }

    run_dcem ();

    debug_message ("Leave process_arguments ().\n");
}


#--------------------------------------------------------------
# main ()
#
#--------------------------------------------------------------
sub main ()
{
    debug_message ("Enter main ()...\n");

    process_arguments ();

    debug_message ("Leave main ().\n");
}


#--------------------------------------------------------------
# START of the perl script.
#
#--------------------------------------------------------------
$|				= 1;

$TRUE				= 1;
$FALSE				= 0;
$DEBUG				= $FALSE;
$SUPPRESS_SPLASH_SCREEN         = $FALSE;

$OS                             = `/bin/uname -s`;
$AIX                            = "AIX";
$LINUX                          = "Linux";

$DJAVA				= "/opt/csm/dcem/bin/djava";
$DISTRIBUTED_CMD		= "";
$JAVA_ARGS			= "";
$VERSION			= "DCEM version 1.3.0.0\n";
$JAVA_CLASS_NAME		= "com.ibm.unix.distrib.framework.ui.DUICommandAppDialog";

$SYSTEM_DEFAULTS                = "-d com.ibm.unix.distrib.distbean.csm.DMXCsmStandaloneSystemDefaults ";
$USER_DEFAULTS                  = "-u com.ibm.unix.distrib.distbean.csm.DMXCsmUserDefaults ";
$FACTORY_DEFAULTS               = "-f /opt/csm/dcem/config/factory_defaults/dcem.cfg ";
$USER_SETTINGS                  = "-s /var/csm/dcem/config/user_settings/dcem.cfg ";
$APPLICATION_MESSAGES           = "-m com.ibm.unix.distrib.distbean.csm.DMXCsmApplicationMessages ";

# Drop the trailing newline character
chop ($OS);

if ($OS eq $AIX)
{
    $SPLASH_SCREEN              = "-i dcem_splash_aix.gif ";
}
else
{
    $SPLASH_SCREEN              = "-i dcem_splash_linux.gif ";
}



#--------------------------------------------------------------
# START of the main function
#
#--------------------------------------------------------------
# Look for the debug option first
if (($ARGV[0] eq "--verbose") || ($ARGV[0] eq "-v")) 
{
    $DEBUG = $TRUE;
    debug_message ("Turning DEBUG ON\n");
    shift (@ARGV); # discard the first argument, i.e. -debug
}

main ();
